"""
Template tag for supporting aa-timezones
"""

# Standard Library
from datetime import datetime

# Django
from django.template.defaulttags import register
from django.utils import formats
from django.utils.safestring import mark_safe
from django.utils.translation import gettext as _

# Alliance Auth (External Libs)
from app_utils.urls import reverse as reverse_url

# AA Forum
from aa_forum.app_settings import aa_timezones_installed


@register.filter
def forum_time(db_datetime: datetime) -> str:
    """
    Convert DB time into a formatted date-time string we use in our templates
    :param db_datetime:
    :type db_datetime:
    :param arg:
    :type arg:
    :return:
    :rtype:
    """

    # If empty, return empty
    if db_datetime in (None, ""):
        return ""

    # Try to format the date for a localised output
    try:
        formatted_date_string = formats.date_format(db_datetime)
    except AttributeError:
        try:
            formatted_date_string = format(db_datetime)
        except AttributeError:
            formatted_date_string = ""

    formatted_time_string = db_datetime.strftime("%H:%M:%S")
    formatted_forum_date = f"{formatted_date_string}, {formatted_time_string}"

    # If `aa-timezones` is installed, add (?) to the date-time string
    # and link to the time zones conversion
    if aa_timezones_installed():
        timestamp_from_db_datetime = int(datetime.timestamp(db_datetime))
        timezones_url = reverse_url(
            "timezones:index", args=[timestamp_from_db_datetime]
        )
        link_title = _("Timezone Conversion")

        return mark_safe(
            f"{formatted_forum_date} "
            f'<sup>(<a href="{timezones_url}" target="_blank" rel="noopener noreferer" '
            f'title="{link_title}"><i class="fas fa-question-circle"></i></a>)</sup>'
        )

    return formatted_forum_date



# Generated by Django 4.0.7 on 2022-09-14 19:08

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aa_forum", "0013_personal_messages_migrate_zero_fix"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="discord_dm_on_new_personal_message",
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 4.0.7 on 2022-08-24 18:02

# Django
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("aa_forum", "0009_add_related_names"),
    ]

    operations = [
        migrations.RenameField(
            model_name="setting",
            old_name="default_max_messages",
            new_name="messages_per_page",
        ),
        migrations.RenameField(
            model_name="setting",
            old_name="default_max_topics",
            new_name="topics_per_page",
        ),
    ]



"""
Test for our helper functions
"""

# Standard Library
from unittest.mock import patch

# Django
from django.contrib.auth.models import Group
from django.test import RequestFactory, TestCase
from django.urls import reverse

# Alliance Auth
from allianceauth.tests.auth_utils import AuthUtils

# Alliance Auth (External Libs)
from app_utils.testing import create_eve_character, create_fake_user

# AA Forum
from aa_forum.forms import NewCategoryForm
from aa_forum.helper.eve_images import get_character_portrait_from_evecharacter
from aa_forum.helper.forms import message_form_errors
from aa_forum.helper.text import get_image_url, string_cleanup
from aa_forum.helper.user import get_main_character_from_user
from aa_forum.models import get_sentinel_user


@patch("aa_forum.helper.forms.messages")
class TestHelperForms(TestCase):
    """
    Testing the forms helper
    """

    def setUp(self) -> None:
        self.factory = RequestFactory()

    def test_should_send_form_errors_as_messages(self, messages):
        """
        Test should send form errors as message
        :param messages:
        :return:
        """

        # given
        form = NewCategoryForm({"name": "Dummy"})
        form.add_error("name", "error message 1")
        form.add_error(None, "error message 2")
        request = self.factory.get(reverse("aa_forum:forum_index"))

        # when
        message_form_errors(request, form)

        # then
        args, _ = messages.error.call_args_list[0]
        self.assertIn("error message 1", args[1])
        args, _ = messages.error.call_args_list[1]
        self.assertIn("error message 2", args[1])

    def test_should_do_nothing_when_form_has_no_errors(self, messages):
        """
        Test should do nothing when form has no errors
        :param messages:
        :return:
        """

        # given
        form = NewCategoryForm({"name": "Dummy"})
        request = self.factory.get(reverse("aa_forum:forum_index"))

        # when
        message_form_errors(request, form)

        # then
        self.assertFalse(messages.error.called)


class TestHelperText(TestCase):
    """
    Testing the text helper
    """

    def test_should_return_cleaned_string(self):
        """
        Test should return a clean/sanitized string
        :return:
        """

        # given
        string = (
            'this is a script test. <script type="text/javascript">alert('
            "'test')</script>and this is style test. <style>.MathJax, "
            ".MathJax_Message, .MathJax_Preview{display: none}</style>end tests."
        )

        # when
        cleaned_string = string_cleanup(string)

        # then
        self.assertIn(
            "this is a script test. and this is style test. end tests.", cleaned_string
        )

    def test_should_return_none_for_get_image_url(self):
        """
        Test should return none for get_image_url
        :return:
        """

        text = (
            "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Mauris "
            "vehicula viverra diam at ultrices. Donec accumsan metus vitae sapien "
            "malesuada, quis molestie quam euismod. Vestibulum malesuada non felis "
            "id pellentesque. Proin in sollicitudin massa. Vestibulum vitae ante "
            "quis dolor placerat feugiat. Praesent id gravida nulla, sed malesuada "
            "arcu. Fusce eu erat feugiat, tincidunt dolor varius, tincidunt velit. "
            "Mauris iaculis elit luctus nunc scelerisque mollis. Etiam imperdiet "
            "mi lacus, at sodales eros aliquet a. Maecenas pretium, odio et "
            "ultricies venenatis, nisl dui volutpat dui, vitae feugiat turpis "
            "dolor vel nisi. Mauris consequat, nulla id mattis ullamcorper, "
            "enim elit convallis erat, eget tempor velit mauris ac nisl. Praesent "
            "sit amet facilisis purus, mattis dictum dolor. Nam id ex sollicitudin "
            "nulla dictum rutrum."
        )

        image_url = get_image_url(text)

        self.assertIsNone(image_url)

    def test_should_return_none_for_get_image_url_2(self):
        """
        Test should return none for get_image_url because it's invalid
        :return:
        """

        text = (
            'Lorem ipsum dolor sit amet, <img src="consectetur adipiscing elit">. Mauris '
            "vehicula viverra diam at ultrices. Donec accumsan metus vitae sapien "
            "malesuada, quis molestie quam euismod. Vestibulum malesuada non felis "
            "id pellentesque. Proin in sollicitudin massa. Vestibulum vitae ante "
            "quis dolor placerat feugiat. Praesent id gravida nulla, sed malesuada "
            "arcu. Fusce eu erat feugiat, tincidunt dolor varius, tincidunt velit. "
            "Mauris iaculis elit luctus nunc scelerisque mollis. Etiam imperdiet "
            "mi lacus, at sodales eros aliquet a. Maecenas pretium, odio et "
            "ultricies venenatis, nisl dui volutpat dui, vitae feugiat turpis "
            "dolor vel nisi. Mauris consequat, nulla id mattis ullamcorper, "
            "enim elit convallis erat, eget tempor velit mauris ac nisl. Praesent "
            "sit amet facilisis purus, mattis dictum dolor. Nam id ex sollicitudin "
            "nulla dictum rutrum."
        )

        image_url = get_image_url(text)

        self.assertIsNone(image_url)

    def test_should_return_first_image_url_for_get_image_url(self):
        """
        Test should return none for get_image_url because it's invalid
        :return:
        """

        text = (
            'Lorem ipsum dolor sit amet, <img src="https://test.de/foobar.jpg">. Mauris '
            "vehicula viverra diam at ultrices. Donec accumsan metus vitae sapien "
            "malesuada, quis molestie quam euismod. Vestibulum malesuada non felis "
            "id pellentesque. Proin in sollicitudin massa. Vestibulum vitae ante "
            "quis dolor placerat feugiat. Praesent id gravida nulla, sed malesuada "
            "arcu. Fusce eu erat feugiat, tincidunt dolor varius, tincidunt velit. "
            "Mauris iaculis elit luctus nunc scelerisque mollis. Etiam imperdiet "
            'mi lacus, at sodales eros <img src="https://test.de/barfoo.jpg"> aliquet '
            "a. Maecenas pretium, odio et ultricies venenatis, nisl dui volutpat dui, "
            "vitae feugiat turpis dolor vel nisi. Mauris consequat, nulla id mattis "
            "ullamcorper, enim elit convallis erat, eget tempor velit mauris ac nisl. "
            "Praesent sit amet facilisis purus, mattis dictum dolor. Nam id ex "
            "sollicitudin nulla dictum rutrum. "
        )

        image_url = get_image_url(text)

        self.assertEqual(image_url, "https://test.de/foobar.jpg")


class TestHelperEveImages(TestCase):
    """
    Testing the eve_images helpers
    """

    def setUp(self) -> None:
        self.user_1001 = create_fake_user(
            1001, "Bruce Wayne", permissions=["aa_forum.basic_access"]
        )

    def test_should_return_character_portrait_url(self):
        """
        Test should return character portrait URL
        :return:
        """

        character = self.user_1001.profile.main_character

        portrait_url = get_character_portrait_from_evecharacter(character=character)
        expected_url = f"https://images.evetech.net/characters/{character.character_id}/portrait?size=32"  # pylint: disable=line-too-long

        self.assertEqual(portrait_url, expected_url)

    def test_should_return_character_portrait_html(self):
        """
        Test should return character portrait HTML image tag
        :return:
        """

        character = self.user_1001.profile.main_character

        portrait_html = get_character_portrait_from_evecharacter(
            character=character, as_html=True
        )
        expected_url = f"https://images.evetech.net/characters/{character.character_id}/portrait?size=32"  # pylint: disable=line-too-long
        expected_html = (
            '<img class="aa-forum-character-portrait img-rounded" '
            f'src="{expected_url}" alt="{character.character_name}" '
            'width="32" height="32">'
        )

        self.assertEqual(portrait_html, expected_html)


class TestGetMainCharacterFromUser(TestCase):
    """
    Tests for get_main_character_from_user
    """

    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()
        cls.group = Group.objects.create(name="Enterprise Crew")

        cls.user_main_character = create_fake_user(
            character_id=1001, character_name="William T. Riker"
        )

        cls.character_without_profile = create_eve_character(
            character_id=1003, character_name="Christopher Pike"
        )

    def test_get_main_character_from_user_should_return_character_name(self):
        """
        Test should return the main character name for a regular user
        :return:
        """

        character_name = get_main_character_from_user(self.user_main_character)

        self.assertEqual(character_name, "William T. Riker")

    def test_get_main_character_from_user_should_return_user_name(self):
        """
        Test should return just the username for a user without a character
        :return:
        """

        user = AuthUtils.create_user("John Doe")

        character_name = get_main_character_from_user(user)

        self.assertEqual(character_name, "John Doe")

    def test_get_main_character_from_user_should_return_sentinel_user(self):
        """
        Test should return "deleted" as username (Sentinel User)
        :return:
        """

        user = get_sentinel_user()

        character_name = get_main_character_from_user(user)

        self.assertEqual(character_name, "deleted")

    def test_get_main_character_from_user_should_return_sentinel_user_for_none(self):
        """
        Test should return "deleted" (Sentinel User) if user is None
        :return:
        """

        user = None

        character_name = get_main_character_from_user(user)

        self.assertEqual(character_name, "deleted")



"""
Search related views
"""

# Standard Library
from functools import reduce
from operator import or_

# Django
from django.contrib.auth.decorators import login_required, permission_required
from django.core.handlers.wsgi import WSGIRequest
from django.db.models import Q
from django.http import HttpResponse
from django.shortcuts import render

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA Forum
from aa_forum import __title__
from aa_forum.constants import SEARCH_STOPWORDS
from aa_forum.helper.pagination import get_paginated_page_object
from aa_forum.models import Board, Message, Setting

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


@login_required
@permission_required("aa_forum.basic_access")
def results(request: WSGIRequest, page_number: int = None) -> HttpResponse:
    """
    Search results view
    :param request:
    :param page_number:
    :return:
    """

    if request.GET:
        search_phrase = request.GET.get("q")
    else:
        search_phrase = ""

    search_results = None
    page_obj = None

    querywords = search_phrase.split()
    search_phrase_terms = [
        word for word in querywords if word.lower() not in SEARCH_STOPWORDS
    ]

    if len(search_phrase_terms) >= 1:
        boards = (
            Board.objects.user_has_access(request.user)
            .distinct()
            .values_list("pk", flat=True)
        )

        search_results = (
            Message.objects.filter(
                reduce(
                    or_,
                    [
                        Q(message_plaintext__icontains=search_term)
                        for search_term in search_phrase_terms
                    ],
                ),
                topic__board__pk__in=boards,
            )
            .select_related(
                "user_created",
                "user_created__profile__main_character",
                "topic",
                "topic__first_message",
                "topic__board",
                "topic__board__category",
            )
            .order_by("-time_modified")
            .distinct()
        )

        page_obj = get_paginated_page_object(
            queryset=search_results,
            items_per_page=Setting.objects.get_setting(
                setting_key=Setting.Field.MESSAGESPERPAGE
            ),
            page_number=page_number,
        )

    context = {
        "search_term": search_phrase,
        "search_results": page_obj,
        "search_results_count": 0 if search_results is None else search_results.count(),
    }

    logger.info(f'{request.user} calling search view, searching for "{search_phrase}".')

    return render(request, "aa_forum/view/search/results.html", context)



# Generated by Django 3.2.11 on 2022-01-24 18:40

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aa_forum", "0003_board_discord_webhook"),
    ]

    operations = [
        migrations.AddField(
            model_name="board",
            name="use_webhook_for_replies",
            field=models.BooleanField(
                default=False,
                help_text="Use this Discord Webhook for replies as well? When activated every reply to any topic in this board will be posted to the defined Discord channel. (Child boards are excluded) Chose wisely! (Default: NO)",
            ),
        ),
    ]



"""
Tests for the admin views
"""

# Standard Library
import json

# Django
from django.test import TestCase
from django.urls import reverse

# AA Forum
from aa_forum.models import Board, Category
from aa_forum.tests.utils import create_fake_user

VIEWS_PATH = "aa_forum.views.admin"


class TestAdminViews(TestCase):
    """
    Test the admin views
    """

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.user = create_fake_user(
            1001,
            "Bruce Wayne",
            permissions=["aa_forum.basic_access", "aa_forum.manage_forum"],
        )

    def test_should_delete_category(self):
        """
        Test should delete a category
        :return:
        """

        # given
        category = Category.objects.create(name="Category")
        self.client.force_login(self.user)

        # when
        res = self.client.get(
            reverse("aa_forum:admin_category_delete", args=[category.pk])
        )

        # then
        self.assertRedirects(res, reverse("aa_forum:admin_categories_and_boards"))
        self.assertFalse(Category.objects.filter(pk=category.pk).exists())

    def test_should_raise_404_when_delete_category_not_found(self):
        """
        Test should raise 404 when category not found on delete
        :return:
        """

        # given
        Category.objects.create(name="Category")
        self.client.force_login(self.user)

        # when
        res = self.client.get(reverse("aa_forum:admin_category_delete", args=[0]))

        # then
        self.assertEqual(res.status_code, 404)

    def test_should_delete_board(self):
        """
        Test should delete board
        :return:
        """

        # given
        category = Category.objects.create(name="Category")
        board = Board.objects.create(name="Board", category=category)
        self.client.force_login(self.user)

        # when
        res = self.client.get(
            reverse("aa_forum:admin_board_delete", args=[category.pk, board.pk])
        )

        # then
        self.assertRedirects(res, reverse("aa_forum:admin_categories_and_boards"))
        self.assertFalse(Board.objects.filter(pk=board.pk).exists())

    def test_should_raise_404_when_delete_board_not_found(self):
        """
        Test should raise 404 when board not found on delete
        :return:
        """

        # given
        category = Category.objects.create(name="Category")
        Board.objects.create(name="Board", category=category)
        self.client.force_login(self.user)

        # when
        res = self.client.get(
            reverse("aa_forum:admin_board_delete", args=[category.pk, 0])
        )

        # then
        self.assertEqual(res.status_code, 404)

    def test_should_save_categories_order(self):
        """
        Test should save category order
        :return:
        """

        # given
        category_1 = Category.objects.create(name="Category 1")
        category_2 = Category.objects.create(name="Category 2")
        self.client.force_login(self.user)

        # when
        res = self.client.post(
            reverse("aa_forum:admin_ajax_category_order"),
            data={
                "categories": json.dumps(
                    [
                        {"catId": category_1.pk, "catOrder": 1},
                        {"catId": category_2.pk, "catOrder": 2},
                    ]
                )
            },
        )

        # then
        self.assertListEqual(res.json(), [{"success": True}])
        category_1.refresh_from_db()
        self.assertEqual(category_1.order, 1)
        category_2.refresh_from_db()
        self.assertEqual(category_2.order, 2)

    def test_should_save_categories_order_and_handle_errors(self):
        """
        Test should save categories order and handle errors
        :return:
        """

        # given
        category_1 = Category.objects.create(name="Category 1")
        self.client.force_login(self.user)

        # when
        res = self.client.post(
            reverse("aa_forum:admin_ajax_category_order"),
            data={
                "categories": json.dumps(
                    [
                        {"catId": category_1.pk, "catOrder": 1},
                        {"catId": 0, "catOrder": 2},
                    ]
                )
            },
        )

        # then
        self.assertListEqual(res.json(), [{"success": True}])
        category_1.refresh_from_db()
        self.assertEqual(category_1.order, 1)

    def test_should_save_boards_order(self):
        """
        Test should save board order
        :return:
        """

        # given
        category = Category.objects.create(name="Category")
        board_1 = Board.objects.create(name="Board 1", category=category)
        board_2 = Board.objects.create(name="Board 2", category=category)
        self.client.force_login(self.user)

        # when
        res = self.client.post(
            reverse("aa_forum:admin_ajax_board_order"),
            data={
                "boards": json.dumps(
                    [
                        {"boardId": board_1.pk, "boardOrder": 1},
                        {"boardId": board_2.pk, "boardOrder": 2},
                    ]
                )
            },
        )

        # then
        self.assertListEqual(res.json(), [{"success": True}])
        board_1.refresh_from_db()
        self.assertEqual(board_1.order, 1)
        board_2.refresh_from_db()
        self.assertEqual(board_2.order, 2)

    def test_should_create_child_board(self):
        """
        Test should create child board
        :return:
        """

        # given
        category = Category.objects.create(name="Category")
        board_1 = Board.objects.create(name="Board 1", category=category)
        self.client.force_login(self.user)

        # when
        res = self.client.get(
            reverse("aa_forum:admin_board_create_child", args=[category.pk, board_1.pk])
        )
        child_board = Board.objects.create(
            name="Child Board 1", category=category, parent_board=board_1
        )

        # then
        self.assertTrue(Board.objects.filter(pk=child_board.pk).exists())
        self.assertRedirects(res, reverse("aa_forum:admin_categories_and_boards"))

    def test_should_save_boards_order_and_handle_errors(self):
        """
        Test should save board order and handle errors
        :return:
        """

        # given
        category = Category.objects.create(name="Category")
        board_1 = Board.objects.create(name="Board 1", category=category)
        self.client.force_login(self.user)

        # when
        res = self.client.post(
            reverse("aa_forum:admin_ajax_board_order"),
            data={
                "boards": json.dumps(
                    [
                        {"boardId": board_1.pk, "boardOrder": 1},
                        {"boardId": 0, "boardOrder": 2},
                    ]
                )
            },
        )

        # then
        self.assertListEqual(res.json(), [{"success": True}])
        board_1.refresh_from_db()
        self.assertEqual(board_1.order, 1)



"""
Helper to handle Discord messaging to users (Discord DM)
"""

# Standard Library
from datetime import datetime

# Third Party
from dhooks_lite import Embed, Footer, Image, Webhook

# Django
from django.utils import timezone

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag
from app_utils.urls import reverse_absolute

# AA Forum
from aa_forum import __title__
from aa_forum.app_settings import (
    allianceauth_discordbot_installed,
    discordproxy_installed,
)
from aa_forum.constants import DISCORD_EMBED_COLOR_MAP
from aa_forum.helper.eve_images import get_character_portrait_from_evecharacter
from aa_forum.helper.text import get_image_url, prepare_message_for_discord
from aa_forum.models import Board, Message, PersonalMessage, Topic

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


def _aadiscordbot_send_private_message(
    user_id: int, level: str, title: str, message: str, embed_message: bool = True
) -> None:
    """
    Try to send a PM to a user on Discord via allianceauth-discordbot
    :param user_id:
    :param level:
    :param title:
    :param message:
    :param embed_message:
    :return:
    """

    if allianceauth_discordbot_installed():
        logger.debug(
            "allianceauth-discordbot is active, trying to send private message"
        )

        # Third Party
        from aadiscordbot.tasks import send_message
        from discord import Embed

        embed = Embed(
            title=str(title),
            description=message,
            color=DISCORD_EMBED_COLOR_MAP.get(level, None),
            timestamp=datetime.now(),
        )

        if embed_message is True:
            send_message(user_id=user_id, embed=embed)
        else:
            send_message(user_id=user_id, message=f"**{title}**\n\n{message}")
    else:
        logger.debug(
            "allianceauth-discordbot is not available on this "
            "system to send the private message"
        )


def _discordproxy_send_private_message(
    user_id: int, level: str, title: str, message: str, embed_message: bool = True
):
    """
    Try to send a PM to a user on Discord via discordproxy
    (fall back to allianceauth-discordbot if needed)
    :param user_id:
    :param level:
    :param title:
    :param message:
    :param embed_message:
    :return:
    """

    # Third Party
    from discordproxy.client import DiscordClient
    from discordproxy.exceptions import DiscordProxyException

    client = DiscordClient()

    try:
        logger.debug("Trying to send a direct message via discordproxy")

        if embed_message is True:
            # Third Party
            from discordproxy.discord_api_pb2 import Embed

            footer = Embed.Footer(text=__title__)
            embed = Embed(
                title=str(title),
                description=message,
                color=DISCORD_EMBED_COLOR_MAP.get(level, None),
                timestamp=timezone.now().isoformat(),
                footer=footer,
            )

            client.create_direct_message(user_id=user_id, embed=embed)
        else:
            client.create_direct_message(
                user_id=user_id, content=f"**{title}**\n\n{message}"
            )
    except DiscordProxyException as ex:
        # Something went wrong with discordproxy
        # Fail silently and try if allianceauth-discordbot is available
        # as a last ditch effort to get the message out to Discord
        logger.debug(
            "Something went wrong with discordproxy, "
            "cannot send a direct message, trying allianceauth-discordbot "
            f"to send the message if available. Error: {ex}"
        )

        _aadiscordbot_send_private_message(
            user_id=user_id,
            level=level,
            title=title,
            message=message,
            embed_message=embed_message,
        )


def send_new_personal_message_notification(
    message: PersonalMessage, embed_message: bool = True
) -> None:
    """
    Send a DM to the recipient of a personal message
    :param message:
    :param embed_message:
    :return:
    """

    # Needs to be imported here, otherwise it's a circular import
    # AA Forum
    from aa_forum.helper.user import get_main_character_from_user, get_user_profile

    recipient_forum_settings = get_user_profile(message.recipient)

    if recipient_forum_settings.discord_dm_on_new_personal_message is True and hasattr(
        message.recipient, "discord"
    ):
        # Get the main characters for sender and recipient
        sender_main_char = get_main_character_from_user(message.sender)
        recipient_main_char = get_main_character_from_user(message.recipient)

        logger.debug(
            f"Sending Discord PM to {recipient_main_char} to "
            "notify about a new personal message"
        )

        message_to_send = prepare_message_for_discord(
            message=message.message, message_length=750
        )
        forum_pm_url = reverse_absolute("aa_forum:personal_messages_inbox")
        dm_level = "info"
        dm_title = "Forum: New Personal Message"
        dm_text = (
            f"Hey {recipient_main_char},\nYou received a new personal message.\n\n"
        )
        dm_text += f"**Sender:** {sender_main_char}\n"
        dm_text += f"**Subject:** {message.subject}\n\n"
        dm_text += f"**Message:**\n{message_to_send}\n\n"
        dm_text += f"[Your Personal Messages]({forum_pm_url})"

        if discordproxy_installed():
            logger.debug("discordproxy seems to be available …")

            _discordproxy_send_private_message(
                user_id=int(message.recipient.discord.uid),
                level=dm_level,
                title=dm_title,
                message=dm_text,
                embed_message=embed_message,
            )
        else:
            # discordproxy not available, try if allianceauth-discordbot is
            # available
            logger.debug(
                "discordproxy not available to send a direct message, "
                "let's see if we can use allianceauth-discordbot if available"
            )

            _aadiscordbot_send_private_message(
                user_id=int(message.recipient.discord.uid),
                level=dm_level,
                title=dm_title,
                message=dm_text,
                embed_message=embed_message,
            )


def send_message_to_discord_webhook(
    board: Board, topic: Topic, message: Message, headline: str
):
    """
    Send a message to a Discord Webhook
    :param board:
    :param topic:
    :param message:
    :param headline:
    :return:
    """

    discord_webhook = Webhook(board.discord_webhook)
    message_to_send = prepare_message_for_discord(message=message.message)
    embed_color = DISCORD_EMBED_COLOR_MAP.get("info", None)
    image_url = get_image_url(message.message)
    author_eve_avatar = get_character_portrait_from_evecharacter(
        character=message.user_created.profile.main_character, size=256
    )
    author_eve_name = message.user_created.profile.main_character.character_name

    if message.pk == message.topic.first_message.pk:
        title = topic.subject
        url = reverse_absolute(
            "aa_forum:forum_topic",
            args=[board.category.slug, board.slug, topic.slug],
        )
    else:
        title = f"Re: {topic.subject}"
        url = reverse_absolute(
            "aa_forum:forum_message",
            args=[board.category.slug, board.slug, topic.slug, message.pk],
        )

    embed = Embed(
        description=message_to_send,
        title=title,
        url=url,
        timestamp=message.time_posted,
        color=embed_color,
        footer=Footer(f"Posted by: {author_eve_name}", author_eve_avatar),
        image=Image(image_url) if image_url else None,
        # thumbnail=Thumbnail(author_eve_avatar),
        # author=Author(
        #     author_eve_name,
        #     # url="https://en.wikipedia.org/wiki/Albert_Einstein",
        #     icon_url=author_eve_avatar,
        # ),
        # fields=[
        #     Field("1st Measurement", "Failed"),
        #     Field("2nd Measurement", "Succeeded"),
        # ],
    )

    discord_webhook.execute(
        headline,
        # username=author_eve_name,
        # avatar_url=author_eve_avatar,
        embeds=[embed],
        wait_for_response=True,
    )



"""
User helper
"""

# Django
from django.contrib.auth.models import User

# AA Forum
from aa_forum.models import UserProfile, get_sentinel_user


def get_user_profile(user: User) -> UserProfile:
    """
    Get a users settings or create them
    :param user:
    :return:
    """

    user_profile, _ = UserProfile.objects.get_or_create(user=user)

    return user_profile


def get_main_character_from_user(user: User) -> str:
    """
    Get the main character from a user
    :param user:
    :type user:
    :return:
    :rtype:
    """

    if user is None:
        sentinel_user = get_sentinel_user()

        return sentinel_user.username

    try:
        return_value = user.profile.main_character.character_name
    except AttributeError:
        return str(user)

    return return_value



"""
Text helper
"""

# Standard Library
import html
import re

# Third Party
from bs4 import BeautifulSoup

# Django
from django.utils.html import strip_tags

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag
from app_utils.urls import site_absolute_url

# AA Forum
from aa_forum import __title__
from aa_forum.constants import DISCORD_EMBED_MESSAGE_LENGTH

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


def verify_image_url(image_url):
    """
    Verify that the passed text is an image URL.

    We're verifying image URLs for inclusion in Slack/Discord Webhook integration,
    which requires a scheme at the beginning (http(s)) and a file extension at the end
    to render correctly. So, a URL which passes verify_url()
    (like example.com/kitten.gif) might not pass this test. If you need to test that
    the URL is both valid AND an image suitable for the Incoming Webhook integration,
    run it through both verify_url() and verify_image_url().
    :param image_url:
    :return:
    """

    return re.match(
        r"(http(s?):)([\/|.|\w|\s|-])*\.(?:gif|jpg|jpeg|png|bmp|webp)", image_url
    )


def get_image_url(text):
    """
    Extract an image url from the passed text. If there are multiple image urls,
    the first to verify will be returned.
    :param text:
    :return:
    """

    soup = BeautifulSoup(text, "html.parser")

    images = soup.findAll("img")

    # The first image to verify we will use
    if images:
        for image in images:
            image__src = image["src"]

            logger.debug(f"Image found: {image__src}")

            if not image__src.startswith(("http://", "https://")):
                logger.debug("Image has no absolute URL, fixing!")

                absolute_site_url = site_absolute_url()
                image__src = f"{absolute_site_url}{image__src}"

            if verify_image_url(image__src):
                logger.debug(f"Image verified: {image__src}")

                return image__src

    logger.debug("No images found.")

    return None


def string_cleanup(string: str) -> str:
    """
    Clean up a string by removing JS, CSS and Head tags
    :param string:
    :return:
    """

    re_head = re.compile(r"<\s*head[^>]*>.*?<\s*/\s*head\s*>", re.S | re.I)
    re_script = re.compile(r"<\s*script[^>]*>.*?<\s*/\s*script\s*>", re.S | re.I)
    re_css = re.compile(r"<\s*style[^>]*>.*?<\s*/\s*style\s*>", re.S | re.I)

    # Strip JS
    string = re_head.sub("", string)
    string = re_script.sub("", string)
    string = re_css.sub("", string)

    return string


def prepare_message_for_discord(
    message: str, message_length=DISCORD_EMBED_MESSAGE_LENGTH
) -> str:
    """
    Preparing the message to be sent with a Discord Webhook

    We have to run strip_tags() twice here.
    1. To remove HTML tags from the text we want to send
    2. After html.unescape() in case that unescaped former escaped HTML tags,
    which now need to be removed as well
    :param message:
    :type message:
    :param message_length:
    :type message_length:
    :return:
    :rtype:
    """

    return strip_tags(
        html.unescape(
            strip_tags(
                message[:message_length] + "…"
                if len(message) > message_length
                else message
            )
        )
    )



"""
Template filter for search views
"""

# Standard Library
import re

# Third Party
from bs4 import BeautifulSoup

# Django
from django.template.defaulttags import register
from django.utils.safestring import mark_safe

# AA Forum
from aa_forum.constants import SEARCH_STOPWORDS


@register.filter
def highlight_search_term(text: str, search_phrase: str) -> str:
    """
    Highlight the search term in search results
    :param text:
    :param search_phrase:
    :return:
    """

    delimiter_search_term_start = "{«}"
    delimiter_search_term_end = "{»}"

    querywords = search_phrase.split()
    search_phrase_terms = [
        word for word in querywords if word.lower() not in SEARCH_STOPWORDS
    ]
    highlighted = text

    for search_term in search_phrase_terms:
        highlighted = re.sub(
            f"(?i)({(re.escape(search_term))})",
            f"{delimiter_search_term_start}\\1{delimiter_search_term_end}",
            highlighted,
        )

    highlighted = BeautifulSoup(highlighted, "html.parser")

    for hyperlink in highlighted.findAll("a"):
        try:
            hyperlink["href"] = (
                hyperlink["href"]
                .replace(delimiter_search_term_start, "")
                .replace(delimiter_search_term_end, "")
            )
        except KeyError:
            # this should never happen, but in case it does, add a dummy href
            hyperlink["href"] = "#"

        try:
            hyperlink["title"] = (
                hyperlink["title"]
                .replace(delimiter_search_term_start, "")
                .replace(delimiter_search_term_end, "")
            )
        except KeyError:
            pass

        try:
            hyperlink["name"] = (
                hyperlink["name"]
                .replace(delimiter_search_term_start, "")
                .replace(delimiter_search_term_end, "")
            )
        except KeyError:
            pass

    for image in highlighted.findAll("img"):
        try:
            image["src"] = (
                image["src"]
                .replace(delimiter_search_term_start, "")
                .replace(delimiter_search_term_end, "")
            )
        except KeyError:
            # this should never happen, but in case it does, add a dummy src
            image["src"] = "#"

        try:
            image["alt"] = (
                image["alt"]
                .replace(delimiter_search_term_start, "")
                .replace(delimiter_search_term_end, "")
            )
        except KeyError:
            pass

        try:
            image["title"] = (
                image["title"]
                .replace(delimiter_search_term_start, "")
                .replace(delimiter_search_term_end, "")
            )
        except KeyError:
            pass

    return mark_safe(
        str(highlighted)
        .replace(
            delimiter_search_term_start, '<span class="aa-forum-search-term-highlight">'
        )
        .replace(delimiter_search_term_end, "</span>")
    )
